@@ -1,5 +1,3 @@
-from datetime import datetime
-
 import django
 from django import forms
 from django.conf import settings
@@ -17,9 +15,8 @@ from django.utils.module_loading import import_string
 from django.utils.translation import gettext_lazy as _
 from django.views.decorators.http import require_POST
 
-from .formats.base_formats import DEFAULT_FORMATS
 from .forms import ConfirmImportForm, ExportForm, ImportForm, export_action_form_factory
-from .resources import modelresource_factory
+from .mixins import BaseExportMixin, BaseImportMixin
 from .results import RowResult
 from .signals import post_export, post_import
 from .tmp_storages import TempFolderStorage
@@ -31,22 +28,18 @@ class ImportExportMixinBase:
         return (app_label, self.model._meta.model_name)
 
 
-class ImportMixin(ImportExportMixinBase):
+class ImportMixin(BaseImportMixin, ImportExportMixinBase):
     """
     Import mixin.
 
     This is intended to be mixed with django.contrib.admin.ModelAdmin
-    https://docs.djangoproject.com/en/2.1/ref/contrib/admin/#modeladmin-objects
+    https://docs.djangoproject.com/en/dev/ref/contrib/admin/
     """
 
     #: template for change_list view
     change_list_template = 'admin/import_export/change_list_import.html'
     #: template for import view
     import_template_name = 'admin/import_export/import.html'
-    #: resource class
-    resource_class = None
-    #: available import formats
-    formats = DEFAULT_FORMATS
     #: import data encoding
     from_encoding = "utf-8"
     skip_admin_log = None
@@ -96,32 +89,6 @@ class ImportMixin(ImportExportMixinBase):
         ]
         return my_urls + urls
 
-    def get_resource_kwargs(self, request, *args, **kwargs):
-        return {}
-
-    def get_import_resource_kwargs(self, request, *args, **kwargs):
-        """Prepares/returns kwargs used when initializing Resource"""
-        return self.get_resource_kwargs(request, *args, **kwargs)
-
-    def get_resource_class(self):
-        """Returns ResourceClass"""
-        if not self.resource_class:
-            return modelresource_factory(self.model)
-        else:
-            return self.resource_class
-
-    def get_import_resource_class(self):
-        """
-        Returns ResourceClass to use for import.
-        """
-        return self.get_resource_class()
-
-    def get_import_formats(self):
-        """
-        Returns available import formats.
-        """
-        return [f for f in self.formats if f().can_import()]
-
     @method_decorator(require_POST)
     def process_import(self, request, *args, **kwargs):
         """
@@ -341,23 +308,19 @@ class ImportMixin(ImportExportMixinBase):
         return super().changelist_view(request, extra_context)
 
 
-class ExportMixin(ImportExportMixinBase):
+class ExportMixin(BaseExportMixin, ImportExportMixinBase):
     """
     Export mixin.
 
     This is intended to be mixed with django.contrib.admin.ModelAdmin
-    https://docs.djangoproject.com/en/2.1/ref/contrib/admin/#modeladmin-objects
+    https://docs.djangoproject.com/en/dev/ref/contrib/admin/
     """
-    #: resource class
-    resource_class = None
     #: template for change_list view
     change_list_template = 'admin/import_export/change_list_export.html'
     #: template for export view
     export_template_name = 'admin/import_export/export.html'
-    #: available export formats
-    formats = DEFAULT_FORMATS
     #: export data encoding
-    to_encoding = "utf-8"
+    to_encoding = None
 
     def get_urls(self):
         urls = super().get_urls()
@@ -380,37 +343,6 @@ class ExportMixin(ImportExportMixinBase):
         codename = get_permission_codename(EXPORT_PERMISSION_CODE, opts)
         return request.user.has_perm("%s.%s" % (opts.app_label, codename))
 
-    def get_resource_kwargs(self, request, *args, **kwargs):
-        return {}
-
-    def get_export_resource_kwargs(self, request, *args, **kwargs):
-        return self.get_resource_kwargs(request, *args, **kwargs)
-
-    def get_resource_class(self):
-        if not self.resource_class:
-            return modelresource_factory(self.model)
-        else:
-            return self.resource_class
-
-    def get_export_resource_class(self):
-        """
-        Returns ResourceClass to use for export.
-        """
-        return self.get_resource_class()
-
-    def get_export_formats(self):
-        """
-        Returns available export formats.
-        """
-        return [f for f in self.formats if f().can_export()]
-
-    def get_export_filename(self, request, queryset, file_format):
-        date_str = datetime.now().strftime('%Y-%m-%d')
-        filename = "%s-%s.%s" % (self.model.__name__,
-                                 date_str,
-                                 file_format.get_extension())
-        return filename
-
     def get_export_queryset(self, request):
         """
         Returns export queryset.
@@ -441,6 +373,8 @@ class ExportMixin(ImportExportMixinBase):
         }
         if django.VERSION >= (2, 1):
             changelist_kwargs['sortable_by'] = self.sortable_by
+        if django.VERSION >= (4, 0):
+            changelist_kwargs['search_help_text'] = self.search_help_text
         cl = ChangeList(**changelist_kwargs)
 
         return cl.get_queryset(request)
@@ -453,9 +387,11 @@ class ExportMixin(ImportExportMixinBase):
         if not self.has_export_permission(request):
             raise PermissionDenied
 
-        resource_class = self.get_export_resource_class()
-        data = resource_class(**self.get_export_resource_kwargs(request)).export(queryset, *args, **kwargs)
+        data = self.get_data_for_export(request, queryset, *args, **kwargs)
         export_data = file_format.export_data(data)
+        encoding = kwargs.get("encoding")
+        if not file_format.is_binary() and encoding:
+            export_data = export_data.encode(encoding)
         return export_data
 
     def get_export_context_data(self, **kwargs):
@@ -476,7 +412,7 @@ class ExportMixin(ImportExportMixinBase):
             ]()
 
             queryset = self.get_export_queryset(request)
-            export_data = self.get_export_data(file_format, queryset, request=request)
+            export_data = self.get_export_data(file_format, queryset, request=request, encoding=self.to_encoding)
             content_type = file_format.get_content_type()
             response = HttpResponse(export_data, content_type=content_type)
             response['Content-Disposition'] = 'attachment; filename="%s"' % (
@@ -503,6 +439,9 @@ class ExportMixin(ImportExportMixinBase):
         extra_context['has_export_permission'] = self.has_export_permission(request)
         return super().changelist_view(request, extra_context)
 
+    def get_export_filename(self, request, queryset, file_format):
+        return super().get_export_filename(file_format)
+
 
 class ImportExportMixin(ImportMixin, ExportMixin):
     """
@@ -553,17 +492,28 @@ class ExportActionMixin(ExportMixin):
             formats = self.get_export_formats()
             file_format = formats[int(export_format)]()
 
-            export_data = self.get_export_data(file_format, queryset, request=request)
+            export_data = self.get_export_data(file_format, queryset, request=request, encoding=self.to_encoding)
             content_type = file_format.get_content_type()
             response = HttpResponse(export_data, content_type=content_type)
             response['Content-Disposition'] = 'attachment; filename="%s"' % (
                 self.get_export_filename(request, queryset, file_format),
             )
             return response
-    export_admin_action.short_description = _(
-        'Export selected %(verbose_name_plural)s')
 
-    actions = admin.ModelAdmin.actions + [export_admin_action]
+    def get_actions(self, request):
+        """
+        Adds the export action to the list of available actions.
+        """
+
+        actions = super().get_actions(request)
+        actions.update(
+            export_admin_action=(
+                ExportActionMixin.export_admin_action,
+                "export_admin_action",
+                _("Export selected %(verbose_name_plural)s"),
+            )
+        )
+        return actions
 
     @property
     def media(self):
