@@ -1,6 +1,7 @@
 from collections import OrderedDict
 
 from django.core.exceptions import NON_FIELD_ERRORS
+from django.utils.encoding import force_str
 from tablib import Dataset
 
 
@@ -32,6 +33,14 @@ class RowResult:
         self.diff = None
         self.import_type = None
         self.raw_values = {}
+        self.object_id = None
+        self.object_repr = None
+
+    def add_instance_info(self, instance):
+        if instance is not None:
+            # Add object info to RowResult (e.g. for LogEntry)
+            self.object_id = getattr(instance, "pk", None)
+            self.object_repr = force_str(instance)
 
 
 class InvalidRow:
