@@ -7,14 +7,14 @@ class atomic_if_using_transaction:
     Replaces code::
 
         if using_transactions:
-            with transaction.atomic():
+            with transaction.atomic(using=using):
                 return something()
         return something()
     """
-    def __init__(self, using_transactions):
+    def __init__(self, using_transactions, using):
         self.using_transactions = using_transactions
         if using_transactions:
-            self.context_manager = transaction.atomic()
+            self.context_manager = transaction.atomic(using=using)
 
     def __enter__(self):
         if self.using_transactions:
