@@ -1,17 +1,37 @@
 import os.path
 from datetime import datetime
+from unittest import mock
+from unittest.mock import MagicMock
 
-from core.admin import AuthorAdmin, BookAdmin, BookResource, CustomBookAdmin
+import chardet
+import tablib
+from core.admin import (
+    AuthorAdmin,
+    BookAdmin,
+    BookResource,
+    CustomBookAdmin,
+    ImportMixin,
+)
 from core.models import Author, Book, Category, EBook, Parent
-from django.contrib.admin.models import LogEntry
+from django.contrib.admin.models import DELETION, LogEntry
 from django.contrib.auth.models import User
+from django.core.exceptions import PermissionDenied
 from django.core.files.uploadedfile import SimpleUploadedFile
+from django.http import HttpRequest
 from django.test.testcases import TestCase
 from django.test.utils import override_settings
 from django.utils.translation import gettext_lazy as _
 from tablib import Dataset
 
+from import_export import formats
+from import_export.admin import (
+    ExportActionMixin,
+    ExportActionModelAdmin,
+    ExportMixin,
+    ImportExportActionModelAdmin,
+)
 from import_export.formats.base_formats import DEFAULT_FORMATS
+from import_export.tmp_storages import TempFolderStorage
 
 
 class ImportExportAdminIntegrationTest(TestCase):
@@ -69,6 +89,38 @@ class ImportExportAdminIntegrationTest(TestCase):
                 1, 0, Book._meta.verbose_name_plural)
         )
 
+    def test_delete_from_admin(self):
+        # test delete from admin site (see #432)
+
+        # create a book which can be deleted
+        b = Book.objects.create(id=1)
+
+        input_format = '0'
+        filename = os.path.join(
+            os.path.dirname(__file__),
+            os.path.pardir,
+            'exports',
+            'books-for-delete.csv')
+        with open(filename, "rb") as f:
+            data = {
+                'input_format': input_format,
+                'import_file': f,
+            }
+            response = self.client.post('/admin/core/book/import/', data)
+        self.assertEqual(response.status_code, 200)
+        confirm_form = response.context['confirm_form']
+        data = confirm_form.initial
+        response = self.client.post('/admin/core/book/process_import/', data,
+                                    follow=True)
+        self.assertEqual(response.status_code, 200)
+
+        # check the LogEntry was created as expected
+        deleted_entry = LogEntry.objects.latest('id')
+        self.assertEqual("delete through import_export", deleted_entry.change_message)
+        self.assertEqual(DELETION, deleted_entry.action_flag)
+        self.assertEqual(b.id, int(deleted_entry.object_id))
+        self.assertEqual("", deleted_entry.object_repr)
+
     @override_settings(TEMPLATE_STRING_IF_INVALID='INVALID_VARIABLE')
     def test_import_mac(self):
         # GET the import form
@@ -239,9 +291,9 @@ class ImportExportAdminIntegrationTest(TestCase):
         # Cause an exception in import_row, but only after import is confirmed,
         # so a failure only occurs when ImportMixin.process_import is called.
         class R(BookResource):
-            def import_obj(self, obj, data, dry_run):
+            def import_obj(self, obj, data, dry_run, **kwargs):
                 if dry_run:
-                    super().import_obj(obj, data, dry_run)
+                    super().import_obj(obj, data, dry_run, **kwargs)
                 else:
                     raise Exception
 
@@ -328,6 +380,111 @@ class ImportExportAdminIntegrationTest(TestCase):
                 1, 0, EBook._meta.verbose_name_plural)
         )
 
+    def test_get_skip_admin_log_attribute(self):
+        m = ImportMixin()
+        m.skip_admin_log = True
+        self.assertTrue(m.get_skip_admin_log())
+
+    def test_get_tmp_storage_class_attribute(self):
+        """Mock dynamically loading a class defined by an attribute"""
+        target = "SomeClass"
+        m = ImportMixin()
+        m.tmp_storage_class = "tmpClass"
+        with mock.patch("import_export.admin.import_string") as mock_import_string:
+            mock_import_string.return_value = target
+            self.assertEqual(target, m.get_tmp_storage_class())
+
+    def test_get_import_data_kwargs_with_form_kwarg(self):
+        """
+        Test that if a the method is called with a 'form' kwarg,
+        then it is removed and the updated dict is returned
+        """
+        request = MagicMock(spec=HttpRequest)
+        m = ImportMixin()
+        kw = {
+            "a": 1,
+            "form": "some_form"
+        }
+        target = {
+            "a": 1
+        }
+        self.assertEqual(target, m.get_import_data_kwargs(request, **kw))
+
+    def test_get_import_data_kwargs_with_no_form_kwarg_returns_empty_dict(self):
+        """
+        Test that if a the method is called with no 'form' kwarg,
+        then an empty dict is returned
+        """
+        request = MagicMock(spec=HttpRequest)
+        m = ImportMixin()
+        kw = {
+            "a": 1,
+        }
+        target = {}
+        self.assertEqual(target, m.get_import_data_kwargs(request, **kw))
+
+    def test_get_context_data_returns_empty_dict(self):
+        m = ExportMixin()
+        self.assertEqual(dict(), m.get_context_data())
+
+    def test_media_attribute(self):
+        """
+        Test that the 'media' attribute of the ModelAdmin class is overridden to include
+        the project-specific js file.
+        """
+        mock_model = mock.MagicMock()
+        mock_site = mock.MagicMock()
+
+        class TestExportActionModelAdmin(ExportActionModelAdmin):
+            def __init__(self):
+                super().__init__(mock_model, mock_site)
+
+        m = TestExportActionModelAdmin()
+        target_media = m.media
+        self.assertEqual('import_export/action_formats.js', target_media._js[-1])
+
+
+class TestImportExportActionModelAdmin(ImportExportActionModelAdmin):
+    def __init__(self, mock_model, mock_site, error_instance):
+        self.error_instance = error_instance
+        super().__init__(mock_model, mock_site)
+
+    def write_to_tmp_storage(self, import_file, input_format):
+        mock_storage = MagicMock(spec=TempFolderStorage)
+
+        mock_storage.read.side_effect = self.error_instance
+        return mock_storage
+
+
+class ImportActionDecodeErrorTest(TestCase):
+    mock_model = mock.Mock(spec=Book)
+    mock_model.__name__ = "mockModel"
+    mock_site = mock.MagicMock()
+    mock_request = MagicMock(spec=HttpRequest)
+    mock_request.POST = {'a': 1}
+    mock_request.FILES = {}
+
+    @mock.patch("import_export.admin.ImportForm")
+    def test_import_action_handles_UnicodeDecodeError(self, mock_form):
+        mock_form.is_valid.return_value = True
+        b_arr = b'\x00\x00'
+        m = TestImportExportActionModelAdmin(self.mock_model, self.mock_site,
+                                                  UnicodeDecodeError('codec', b_arr, 1, 2, 'fail!'))
+        res = m.import_action(self.mock_request)
+        self.assertEqual(
+            "<h1>Imported file has a wrong encoding: \'codec\' codec can\'t decode byte 0x00 in position 1: fail!</h1>",
+            res.content.decode())
+
+    @mock.patch("import_export.admin.ImportForm")
+    def test_import_action_handles_error(self, mock_form):
+        mock_form.is_valid.return_value = True
+        m = TestImportExportActionModelAdmin(self.mock_model, self.mock_site,
+                                                  ValueError("fail"))
+        res = m.import_action(self.mock_request)
+        self.assertRegex(
+            res.content.decode(),
+            r"<h1>ValueError encountered while trying to read file: .*</h1>")
+
 
 class ExportActionAdminIntegrationTest(TestCase):
 
@@ -366,3 +523,88 @@ class ExportActionAdminIntegrationTest(TestCase):
         }
         response = self.client.post('/admin/core/category/', data)
         self.assertEqual(response.status_code, 302)
+
+    def test_get_export_data_raises_PermissionDenied_when_no_export_permission_assigned(self):
+        request = MagicMock(spec=HttpRequest)
+
+        class TestMixin(ExportMixin):
+            model = Book
+
+            def has_export_permission(self, request):
+                return False
+        m = TestMixin()
+        with self.assertRaises(PermissionDenied):
+            m.get_export_data('0', Book.objects.none(), request=request)
+
+
+class TestExportEncoding(TestCase):
+    mock_request = MagicMock(spec=HttpRequest)
+    mock_request.POST = {'file_format': 0}
+
+    class TestMixin(ExportMixin):
+        def __init__(self, test_str=None):
+            self.test_str = test_str
+
+        def get_data_for_export(self, request, queryset, *args, **kwargs):
+            dataset = Dataset(headers=["id", "name"])
+            dataset.append([1, self.test_str])
+            return dataset
+
+        def get_export_queryset(self, request):
+            return list()
+
+        def get_export_filename(self, request, queryset, file_format):
+            return "f"
+
+    def setUp(self):
+        self.file_format = formats.base_formats.CSV()
+        self.export_mixin = self.TestMixin(test_str="teststr")
+
+    def test_to_encoding_not_set_default_encoding_is_utf8(self):
+        self.export_mixin = self.TestMixin(test_str="teststr")
+        data = self.export_mixin.get_export_data(self.file_format, list(), request=self.mock_request)
+        csv_dataset = tablib.import_set(data)
+        self.assertEqual("teststr", csv_dataset.dict[0]["name"])
+
+    def test_to_encoding_set(self):
+        self.export_mixin = self.TestMixin(test_str="ハローワールド")
+        data = self.export_mixin.get_export_data(self.file_format, list(), request=self.mock_request, encoding="shift-jis")
+        encoding = chardet.detect(bytes(data))["encoding"]
+        self.assertEqual("SHIFT_JIS", encoding)
+
+    def test_to_encoding_set_incorrect(self):
+        self.export_mixin = self.TestMixin()
+        with self.assertRaises(LookupError):
+            self.export_mixin.get_export_data(self.file_format, list(), request=self.mock_request, encoding="bad-encoding")
+
+    def test_to_encoding_not_set_for_binary_file(self):
+        self.export_mixin = self.TestMixin(test_str="teststr")
+        self.file_format = formats.base_formats.XLSX()
+        data = self.export_mixin.get_export_data(self.file_format, list(), request=self.mock_request)
+        binary_dataset = tablib.import_set(data)
+        self.assertEqual("teststr", binary_dataset.dict[0]["name"])
+
+    @mock.patch("import_export.admin.ImportForm")
+    def test_export_action_to_encoding(self, mock_form):
+        mock_form.is_valid.return_value = True
+        self.export_mixin.to_encoding = "utf-8"
+        with mock.patch("import_export.admin.ExportMixin.get_export_data") as mock_get_export_data:
+            self.export_mixin.export_action(self.mock_request)
+            encoding_kwarg = mock_get_export_data.call_args_list[0][1]["encoding"]
+            self.assertEqual("utf-8", encoding_kwarg)
+
+    @mock.patch("import_export.admin.ImportForm")
+    def test_export_admin_action_to_encoding(self, mock_form):
+        class TestExportActionMixin(ExportActionMixin):
+            def get_export_filename(self, request, queryset, file_format):
+                return "f"
+
+        self.mock_request.POST = {'file_format': '1'}
+
+        self.export_mixin = TestExportActionMixin()
+        self.export_mixin.to_encoding = "utf-8"
+        mock_form.is_valid.return_value = True
+        with mock.patch("import_export.admin.ExportMixin.get_export_data") as mock_get_export_data:
+            self.export_mixin.export_admin_action(self.mock_request, list())
+            encoding_kwarg = mock_get_export_data.call_args_list[0][1]["encoding"]
+            self.assertEqual("utf-8", encoding_kwarg)
\ No newline at end of file
