@@ -1,6 +1,8 @@
 import os
 import sys
 
+import django
+
 INSTALLED_APPS = [
     'django.contrib.admin',
     'django.contrib.auth',
@@ -48,6 +50,9 @@ TEMPLATES = [
     },
 ]
 
+if django.VERSION >= (3, 2):
+    DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'
+
 if os.environ.get('IMPORT_EXPORT_TEST_TYPE') == 'mysql-innodb':
     IMPORT_EXPORT_USE_TRANSACTIONS = True
     DATABASES = {
@@ -103,5 +108,4 @@ LOGGING = {
     }}
 
 
-# USE_TZ = True
-# TIME_ZONE = 'Europe/Ljubljana'
+USE_TZ = False
